#include <windows.h>
#include "process_buffer.h"

//////////////////////////////////////////////////////////////////////////
//
//   
//
//////////////////////////////////////////////////////////////////////////

CProcessBuffer::CProcessBuffer(HANDLE hProcess)
{
	DuplicateHandle(
		GetCurrentProcess(), 
		hProcess, 
		GetCurrentProcess(), 
		&m_hProcess, 
		0, 
		FALSE, 
		DUPLICATE_SAME_ACCESS);

	m_size = 0;
	m_buffer = 0;
}

CProcessBuffer::~CProcessBuffer()
{
	VirtualFreeEx(m_hProcess, m_buffer, m_size, MEM_RELEASE);
	CloseHandle(m_hProcess);
}

//////////////////////////////////////////////////////////////////////////
//
//   
//
//////////////////////////////////////////////////////////////////////////

bool CProcessBuffer::alloc(DWORD s)
{
	free();

	m_buffer = (char*)VirtualAllocEx(m_hProcess, 0, s, MEM_COMMIT, PAGE_READWRITE);

	if (!m_buffer)
	{
		return false;
	}

	m_size = s;
	return true;
}

void CProcessBuffer::free()
{
	if (!m_buffer)
	{
		return;
	}

	VirtualFreeEx(m_hProcess, m_buffer, m_size, MEM_RELEASE);
	m_buffer = 0;
	m_size = 0;
}

//////////////////////////////////////////////////////////////////////////
//
//   
//
//////////////////////////////////////////////////////////////////////////

bool CProcessBuffer::write(DWORD offset, const void* buff, int size)
{
	if (offset + size > m_size)
	{
		return false;
	}

	return WriteProcessMemory(m_hProcess, m_buffer + offset, buff, size, 0);
}

bool CProcessBuffer::read(DWORD offset, void* buff, int size)
{
	if (offset + size > m_size)
	{
		return false;
	}

	return ReadProcessMemory(m_hProcess, m_buffer + offset, buff, size, 0);
}

//////////////////////////////////////////////////////////////////////////
